package cn.fw.hestia.common.utils

import org.slf4j.LoggerFactory
import java.sql.Timestamp
import java.text.SimpleDateFormat
import java.time.*
import java.time.temporal.TemporalAdjusters
import java.util.*

/**
 * 日期处理工具
 *
 *
 *
 * @author kurisu
 */

object DateUtil {
    private val sdf = SimpleDateFormat("yyyy-MM-dd")
    private val log = LoggerFactory.getLogger(this::class.java)
    fun parse(date: String?): Date? {
        try {
            return sdf.parse(date)
        } catch (e: Exception) {
            log.error("解析失败", e)
        }
        return null
    }

    /**
     * 开始日期处理
     */
    fun startDate(date: Date?): Date? {
        return date?.also {
            localDateTime2Date(
                date2LocalDate(it)?.atStartOfDay()
            )
        }
    }

    /**
     * 最结束日期处理
     */
    fun endDate(date: Date?): Date? {
        if (date == null) {
            return null
        }
        return localDateTime2Date(
            date2LocalDate(date)?.plusDays(1)?.atStartOfDay()
        )
    }

    /**
     * 处理日期，保留年月日
     */
    fun date2LocalDate(date: Date?): LocalDate? {
        if (date == null) {
            return null
        }
        return LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault()).toLocalDate()
    }

    fun date2LocalTime(date: Date?): LocalTime? {
        if (date == null) {
            return null
        }
        return LocalTime.of(date.hours, date.minutes, date.seconds)
    }

    fun date2LocalDateTime(date: Date): LocalDateTime {
        val instant: Instant = date.toInstant()
        val zoneId: ZoneId = ZoneId.systemDefault()
        return instant.atZone(zoneId).toLocalDateTime()
    }

    /**
     * convert LocalDateTime to Date
     */
    fun localDateTime2Date(dateTime: LocalDateTime?): Date? {
        return dateTime?.let {
            Date.from(it.atZone(ZoneId.systemDefault()).toInstant())
        }
    }

    /**
     * convert LocalDate to Date
     */
    fun localDate2Date(localDate: LocalDate?): Date? {
        if (localDate == null) {
            return null
        }
        return Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant())
    }

    fun getBeginInTime(inTime: Date?): Date? {
        if (inTime == null) {
            return null
        }
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        c.set(Calendar.HOUR_OF_DAY, 0)
        c.set(Calendar.MINUTE, 0)
        c.set(Calendar.SECOND, 0)
        return c.time
    }

    fun getEndInTime(inTime: Date?): Date? {
        if (inTime == null) {
            return null
        }
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        c.set(Calendar.HOUR_OF_DAY, 23)
        c.set(Calendar.MINUTE, 59)
        c.set(Calendar.SECOND, 59)
        return c.time
    }

    /**
     * 得到本周周一
     *
     * @return
     */
    fun getMondayOfThisWeek(inTime: Date?): Date {
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        var day_of_week: Int = c.get(Calendar.DAY_OF_WEEK) - 1
        if (day_of_week == 0) {
            day_of_week = 7
        }
        c.add(Calendar.DATE, -day_of_week + 1)
        return c.time
    }

    /**
     * 得到本周周日
     *
     * @return
     */
    fun getSundayOfThisWeek(inTime: Date?): Date {
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        var day_of_week: Int = c.get(Calendar.DAY_OF_WEEK) - 1
        if (day_of_week == 0) {
            day_of_week = 7
        }
        c.add(Calendar.DATE, -day_of_week + 7)
        return c.time
    }

    /**
     * 得到月初
     *
     * @return
     */
    fun getMonthFirstDay(inTime: Date?): Date {
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        c.add(Calendar.MONTH, 0)
        c.set(Calendar.DAY_OF_MONTH, 1)
        return c.time
    }

    val monthFirstDay: Date?
        /**
         * 获取当月月初时间
         */
        get() = localDate2Date(
            LocalDateTime.now()
                .with(TemporalAdjusters.firstDayOfMonth()).toLocalDate()
        )

    /**
     * 得到月末
     *
     * @return
     */
    fun getMonthEndDay(inTime: Date?): Date {
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        c.add(Calendar.MONTH, 1)
        c.set(Calendar.DAY_OF_MONTH, 0)
        return c.time
    }

    val monthEndDay: Date?
        /**
         * 得到月末
         *
         * @return
         */
        get() {
            return localDate2Date(
                LocalDateTime.now()
                    .with(TemporalAdjusters.lastDayOfMonth()).toLocalDate()
            )
        }

    /**
     * 得到年初
     *
     * @return
     */
    fun getYearFirstDay(inTime: Date?): Date? {
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        val x: Int = c.get(Calendar.YEAR)
        try {
            return sdf.parse("$x-01-01")
        } catch (_: Exception) {
        }
        return null
    }

    /**
     * 得到年末
     *
     * @return
     */
    fun getYearEndDay(inTime: Date?): Date? {
        val c: Calendar = Calendar.getInstance()
        c.time = inTime
        val x: Int = c.get(Calendar.YEAR)
        try {
            return sdf.parse("$x-12-31")
        } catch (_: Exception) {
        }
        return null
    }

    fun getThisHourStartTime(time: Date): Date {
        val thisHourStart: LocalDateTime = LocalDateTime.of(
            date2LocalDateTime(time).year,
            date2LocalDateTime(time).month,
            date2LocalDateTime(time).dayOfMonth,
            date2LocalDateTime(time).hour,
            0, 0
        )
        return localDateTime2Date(thisHourStart)!!
    }

    fun getThisHourEndTime(time: Date): Date {
        val thisHourEnd: LocalDateTime = LocalDateTime.of(
            date2LocalDateTime(time).year,
            date2LocalDateTime(time).month,
            date2LocalDateTime(time).dayOfMonth,
            date2LocalDateTime(time).hour,
            59, 59
        )
        return localDateTime2Date(thisHourEnd)!!
    }

    val currentDayStartTime: Date
        /**
         * 获取当天起始时间
         *
         * @return 当天起始时间
         */
        get() {
            val now: LocalDateTime = LocalDateTime.now()
            val start: LocalDateTime = LocalDateTime.of(now.year, now.month, now.dayOfMonth, 0, 0, 0)
            val zone: ZoneId = ZoneId.systemDefault()
            val instant: Instant = start.atZone(zone).toInstant()
            return Date.from(instant)
        }

    val currentDayEndTime: Date
        /**
         * 获取当天结束时间
         *
         * @return 当天结束时间
         */
        get() {
            val now: LocalDateTime = LocalDateTime.now()
            val end: LocalDateTime = LocalDateTime.of(now.year, now.month, now.dayOfMonth, 23, 59, 59)
            val zone: ZoneId = ZoneId.systemDefault()
            val instant: Instant = end.atZone(zone).toInstant()
            return Date.from(instant)
        }


    /**
     * 获取短时间字符串
     *
     * @param date 需要转换格式的日期
     * @return 返回短时间字符串格式 yyyy-MM-dd
     */
    fun getStringDateShort(date: Date?): String? {
        if (date == null) {
            return null
        }
        val dateString: String = sdf.format(date)
        return dateString
    }

    /**
     * 获取时间字符串
     *
     * @param date 需要转换格式的日期
     * @return 返回短时间字符串格式 yyyy-MM-dd HH:mm:ss
     */
    fun getFullDateString(date: Date?): String {
        if (date == null) {
            return ""
        }
        val format = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
        return format.format(date)
    }

    /**
     * 获取时间字符串
     *
     * @param date 需要转换格式的日期
     * @return 返回短时间字符串格式 yyyy-MM
     */
    fun getMonthString(date: Date?): String {
        if (date == null) {
            return ""
        }
        val format = SimpleDateFormat("yyyy-MM")
        return format.format(date)
    }

    /**
     * 获取时间字符串
     *
     * @param date    需要转换格式的日期
     * @param pattern 时间字符串格式
     * @return 返回时间字符串
     */
    fun getFormatString(date: Date?, pattern: String?): String {
        if (date == null) {
            return ""
        }
        val format = SimpleDateFormat(pattern)
        return format.format(date)
    }

    val nextMonthStartTime: Date?
        /**
         * 获取下月月初时间
         */
        get() {
            return localDate2Date(
                LocalDateTime.now()
                    .with(TemporalAdjusters.lastDayOfMonth()).plusDays(1L).toLocalDate()
            )
        }

    /**
     * @param offsetYear
     * @return 当前时间 + offsetYear
     */
    fun getNowExpiredYear(offsetYear: Int): Timestamp {
        val now: Calendar = Calendar.getInstance()
        now.add(Calendar.YEAR, offsetYear)
        return Timestamp(now.time.time)
    }

    /**
     * @param offset
     * @return 当前时间 + offsetMonth
     */
    fun getNowExpiredMonth(offset: Int): Timestamp {
        val now: Calendar = Calendar.getInstance()
        now.add(Calendar.MONTH, offset)
        return Timestamp(now.time.time)
    }

    /**
     * @param offset
     * @return 当前时间 + offsetDay
     */
    fun getNowExpiredDay(offset: Int): Timestamp {
        val now: Calendar = Calendar.getInstance()
        now.add(Calendar.DATE, offset)
        return Timestamp(now.time.time)
    }

    /**
     * @param offset
     * @return 当前时间 + offsetDay
     */
    fun getNowExpiredHour(offset: Int): Timestamp {
        val now: Calendar = Calendar.getInstance()
        now.add(Calendar.HOUR, offset)
        return Timestamp(now.time.time)
    }

    /**
     * @param offsetSecond
     * @return 当前时间 + offsetSecond
     */
    fun getNowExpiredSecond(offsetSecond: Int): Timestamp {
        val now: Calendar = Calendar.getInstance()
        now.add(Calendar.SECOND, offsetSecond)
        return Timestamp(now.time.time)
    }

    /**
     * @param offset
     * @return 当前时间 + offset
     */
    fun getNowExpiredMinute(offset: Int): Timestamp {
        val now: Calendar = Calendar.getInstance()
        now.add(Calendar.MINUTE, offset)
        return Timestamp(now.time.time)
    }

    /**
     * @param offset
     * @return 指定时间 + offsetDay
     */
    fun getExpiredDay(givenDate: Date?, offset: Int): Timestamp {
        val date: Calendar = Calendar.getInstance()
        date.time = givenDate
        date.add(Calendar.DATE, offset)
        return Timestamp(date.time.time)
    }

    /**
     * 实现ORACLE中ADD_MONTHS函数功能
     *
     * @param offset
     * @return 指定时间 + offsetMonth
     */
    fun getExpiredMonth(givenDate: Date?, offset: Int): Timestamp {
        val date: Calendar = Calendar.getInstance()
        date.time = givenDate
        date.add(Calendar.MONTH, offset)
        return Timestamp(date.time.time)
    }

    /**
     * @param
     * @return 指定时间 + offsetYear
     */
    fun getExpiredYear(givenDate: Date?, offsetYear: Int): Timestamp {
        val date: Calendar = Calendar.getInstance()
        date.time = givenDate
        date.add(Calendar.YEAR, offsetYear)
        return Timestamp(date.time.time)
    }

    /**
     * @param second
     * @return 指定时间 + offsetSecond
     */
    fun getExpiredSecond(givenDate: Date?, second: Int): Timestamp {
        val date: Calendar = Calendar.getInstance()
        date.time = givenDate
        date.add(Calendar.SECOND, second)
        return Timestamp(date.time.time)
    }

    /**
     * 计算时间差
     *
     * @param givenDate 日期
     * @param offset    2
     * @param type      日期字段类型
     * @return
     */
    fun getExpired(givenDate: Date, offset: Int, type: Int): Timestamp {
        val date: Calendar = Calendar.getInstance()
        date.time = givenDate
        date.add(type, offset)
        return Timestamp(date.time.time)
    }

    /**
     * 计算日期差
     *
     * @param d1   开始日期
     * @param d2   截止日期
     * @param type 类型
     * @return 计算结果
     */
    fun sub(d1: Date?, d2: Date?, type: String): Int? {
        if (d1 == null || d2 == null) {
            return null
        }
        var result = 0L
        val DAY: Long = 24 * 60 * 60 * 1000L
        val sub: Long = d1.time - d2.time
        val daysub: Long = (sub / DAY)
        val y1: Long = d1.year.toLong()
        val y2: Long = d2.year.toLong()
        val m1: Long = d1.month.toLong()
        val m2: Long = d2.month.toLong()
        val monthsub: Long = (y1 - y2) * 12 + (m1 - m2)
        //月
        if (("m" == type)) {
            result = monthsub
            //年
        } else if (("y" == type)) {
            result = monthsub / 12
            //天
        } else if (("d" == type)) {
            result = daysub
        }
        return result.toInt()
    }

    /**
     * 判断是否是凌晨
     *
     * @param date
     * @return
     */
    fun isBeforeDawn(date: Date): Boolean {
        if (Objects.isNull(date)) {
            return false
        }
        val dateTime: LocalDateTime = date2LocalDateTime(date)
        val sdf = SimpleDateFormat("HH")
        val str: String = sdf.format(localDateTime2Date(dateTime))
        val a: Int = str.toInt()
        return a in 0..6
    }

    /**
     * 判断是否是上午
     *
     * @param date
     * @return
     */
    fun isMorning(date: Date): Boolean {
        if (!Objects.nonNull(date)) {
            return false
        }
        val dateTime: LocalDateTime = date2LocalDateTime(date)
        val sdf = SimpleDateFormat("HH")
        val str: String = sdf.format(localDateTime2Date(dateTime))
        val a: Int = str.toInt()
        return a in 7..12
    }

    /**
     * 判断是否是下午
     *
     * @param date
     * @return
     */
    fun isAfternoon(date: Date): Boolean {
        if (Objects.isNull(date)) {
            return false
        }
        val dateTime: LocalDateTime = date2LocalDateTime(date)
        val df = SimpleDateFormat("HH")
        val str: String = df.format(localDateTime2Date(dateTime))
        val a: Int = str.toInt()
        return a in 13..18
    }

    /**
     * 判断是否是晚上
     *
     * @param date
     * @return
     */
    fun isEvening(date: Date): Boolean {
        if (Objects.isNull(date)) {
            return false
        }
        val dateTime: LocalDateTime = date2LocalDateTime(date)
        val df = SimpleDateFormat("HH")
        val str: String = df.format(localDateTime2Date(dateTime))
        val a: Int = str.toInt()
        return a in 19..23
    }
}
