package cn.fw.hestia.common.utils

import java.io.UnsupportedEncodingException
import java.util.*
import java.util.regex.Pattern

/**
 * 字符串工具
 *
 * @author: kurisu
 * @version: 1.0
 */

object StringUtils {

    /**
     * 空字符串
     */
    const val EMPTY: String = ""

    /**
     * 特殊字符正则表达式
     */
    const val regEx: String = "[`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]"

    const val regNo: String = "^[-+]?\\d*$"

    /**
     * 首字母小写
     *
     * @param s String
     * @return String
     */
    fun firstCharLowerCase(s: String): String {
        if (isValid(s)) {
            return s.substring(0, 1).lowercase(Locale.getDefault()) + s.substring(1)
        }
        return s
    }

    /**
     * 删除前缀
     *
     * @param s
     * @param prefix
     * @return
     */
    fun removePrefix(s: String, prefix: String): String {
        val index = s.indexOf(prefix)
        return if (index == 0) s.substring(prefix.length) else s
    }

    /**
     * 删除后缀
     *
     * @param s
     * @param suffix
     * @return
     */
    fun removeSuffix(s: String, suffix: String): String {
        return if (s.endsWith(suffix)) s.substring(0, s.length - suffix.length) else s
    }

    /**
     * 首字母大写
     *
     * @param s String
     * @return String
     */
    fun firstCharUpperCase(s: String): String {
        if (isValid(s)) {
            return s.substring(0, 1).uppercase(Locale.getDefault()) + s.substring(1)
        }
        return s
    }

    /**
     * 检查对象是否有效 obj != null && obj.toString().length() > 0
     *
     * @param obj
     * @return boolean
     */
    fun isValid(obj: Any?): Boolean {
        return obj?.run {
            toString().isNotEmpty()
        } ?: false
    }

    /**
     * 是否是空的
     *
     * @param obj
     * @return
     */
    fun isEmpty(obj: Any?): Boolean {
        return obj?.run {
            toString().isEmpty()
        } ?: true
    }

    /**
     * 是否是数字
     *
     * @param obj
     * @return
     */
    fun isNumber(obj: Any): Boolean {
        if (isEmpty(obj)) {
            return false
        }
        return Pattern.compile(regNo).matcher(obj.toString()).matches()
    }

    /**
     * 转化为String对象
     *
     * @param obj
     * @return boolean
     */
    fun asString(obj: Any?): String {
        return obj?.toString() ?: ""
    }

    /**
     * 返回其中一个有效的对象 value != null && value.toString().length() > 0
     *
     * @param values
     */
    fun tryThese(vararg values: Any?): String {
        for (o in values) {
            val value = asString(o)
            if (value.isNotEmpty()) {
                return value
            }
        }
        return ""
    }

    /**
     * EL表达式提供的定义方法
     *
     * @param v1
     * @param v2
     * @return
     */
    fun tryThese(v1: String, v2: String): String {
        return tryThese(*arrayOf<Any>(v1, v2))
    }

    /**
     * 连接字符串
     *
     * @param list
     * @param split
     * @return 字符串
     */
    fun <T> join(list: Array<T>?, split: String?): String? {
        return join(list, split, "")
    }

    /**
     * 连接字符串
     *
     * @param list
     * @param split
     * @return 字符串
     */
    fun <T> join(list: Array<T>?, split: String?, wrap: String?): String? {
        if (list == null) {
            return null
        }
        val s = StringBuilder(128)
        for (i in list.indices) {
            if (i > 0) {
                s.append(split)
            }
            s.append(wrap).append(list[i]).append(wrap)
        }
        return s.toString()
    }

    /**
     * 包裹字符串 id:12, {, } 输出 {id:12}
     *
     * @param input 输入串
     * @param begin {
     * @param end   }
     * @return String
     */
    fun wrap(begin: String, input: String, end: String): String {
        var inputStr = input
        if (!inputStr.startsWith(begin)) {
            inputStr = begin + input
        }
        if (!inputStr.endsWith(end)) {
            inputStr += end
        }
        return inputStr
    }

    /**
     * 取得匹配的字符串
     *
     * @param input
     * @param regex
     * @return
     */
    fun matchs(input: String, regex: String): List<String> {
        return matchs(input, regex, 0)
    }

    /**
     * 取得匹配的字符串
     *
     * @param input
     * @param regex
     * @return
     */
    fun matchs(input: CharSequence, regex: String, group: Int): List<String> {
        val pattern = Pattern.compile(regex)
        val match = pattern.matcher(input)
        val matches: MutableList<String> = ArrayList()
        while (match.find()) {
            matches.add(match.group(group))
        }
        return matches
    }

    /**
     * 找到匹配的第一个字符串
     *
     * @param input
     * @param regex
     * @param group
     * @return
     */
    fun matchFirst(input: String, regex: String, group: Int): String? {
        val matches = matchs(input, regex, group)
        return if (matches.isEmpty()) null else matches[0]
    }

    /**
     * 截取指定长度字符串
     *
     * @return
     */
    fun getShorterString(str: String, maxLength: Int): String {
        return getShorterString(str, "...", maxLength)
    }

    /**
     * 截取指定长度字符串
     *
     * @param input
     * @param tail
     * @param length
     * @return
     */
    fun getShorterString(input: String, tail: String?, length: Int): String {
        var tailStr = tail
        tailStr = if (isValid(tailStr)) tailStr else ""
        val buffer = StringBuffer(512)
        try {
            val len = input.toByteArray(charset("GBK")).size
            if (len > length) {
                var ln = 0
                var i = 0
                while (ln < length) {
                    val temp = input.substring(i, i + 1)
                    if (temp.toByteArray(charset("GBK")).size == 2) {
                        ln += 2
                    } else {
                        ln++
                    }

                    if (ln <= length) {
                        buffer.append(temp)
                    }
                    i++
                }
            } else {
                return input
            }
            buffer.append(tailStr)
        } catch (ignored: UnsupportedEncodingException) {
        }
        return buffer.toString()
    }

    /**
     * 取得GBK编码
     *
     * @return
     */
    fun getBytesString(input: String, code: String): String {
        try {
            val b = input.toByteArray(charset(code))
            return b.contentToString()
        } catch (e: UnsupportedEncodingException) {
            return code.hashCode().toString()
        }
    }

    /**
     * 转换格式 CUST_INFO_ID - > custInfoId
     *
     * @param input
     * @return
     */
    fun getFieldString(input: String?): String? {
        return input?.run {
            val field = lowercase(Locale.getDefault())
            val values = field.split("_".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
            val b = StringBuffer(length)
            for (i in values.indices) {
                if (i == 0) {
                    b.append(values[i])
                } else {
                    b.append(firstCharUpperCase(values[i]))
                }
            }
            b.toString()
        }
    }

    /**
     * 转换格式 CUST_INFO_ID - > custInfoId
     *
     * @param columnName
     * @return
     */
    fun toFieldName(columnName: String?): String? {
        return getFieldString(columnName)
    }

    /**
     * 转换格式 custInfoId - > CUST_INFO_ID
     *
     * @param field
     * @return
     */
    fun toColumnName(field: String?): String? {
        if (field == null) {
            return null
        }
        val b = StringBuffer(field.length + 3)
        for (i in field.indices) {
            val char1 = field[i]
            if (Character.isUpperCase(char1) && i != 0) {
                b.append("_")
            }
            b.append(char1)
        }
        return b.toString()
    }

    /**
     * 转化为JSON值
     *
     * @param value
     * @return
     * @throws IOException
     */
    fun toJsonValue(value: Any): String {
        return if (value is Number) {
            value.toString()
        } else {
            "'$value'"
        }
    }

    /**
     * 字符串转化为UUID
     *
     * @param value
     * @return
     */
    fun toUUID(value: String?): String {
        if (value == null) {
            throw RuntimeException("value is null!")
        }
        return UUID.nameUUIDFromBytes(value.toByteArray()).toString()
    }

    /**
     * 获取Style样式中样式的值
     *
     * @param styleString
     * @param styleName
     * @return 相应的值
     */
    fun getStyleValue(styleString: String, styleName: String?): String {
        val styles = styleString.split(";".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
        for (s in styles) {
            val tempValue = s.trim { it <= ' ' }
            if (tempValue.startsWith(styleName!!)) {
                val style = tempValue.split(":".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                return style[1]
            }
        }
        return ""
    }

    /**
     * 生成重复次字符
     *
     * @param character
     * @param repeat
     * @return
     */
    fun getRepeat(character: String, repeat: Int): String {
        return repeat(character, repeat, "")
    }

    /**
     * 生成重复次字符
     *
     * @param character
     * @param repeat
     * @return
     */
    fun repeat(character: String, repeat: Int, split: String?): String {
        val s = StringBuilder(character.length * repeat)
        for (i in 0 until repeat) {
            if (i != 0) {
                s.append(split ?: "")
            }
            s.append(character)
        }
        return s.toString()
    }

    /**
     * 取得长度
     *
     * @param text
     * @return
     */
    fun length(text: String): Int {
        var len = text.length
        try {
            len = text.toByteArray(charset("GBK")).size //SQLServer数据库用的GBK编码
        } catch (ignored: UnsupportedEncodingException) {
        }
        return len
    }

    /**
     * 字符串替换函数
     *
     * @param data 字符串
     * @param data from  旧值
     * @param to   from  新值
     */
    fun replaceString(data: String, from: String, to: String?): String {
        val buf = StringBuffer(data.length)
        var pos: Int
        var i = 0
        while ((data.indexOf(from, i).also { pos = it }) != -1) {
            buf.append(data.substring(i, pos)).append(to)
            i = pos + from.length
        }
        buf.append(data.substring(i))
        return buf.toString()
    }

    /**
     * 转义特殊字符
     *
     * @param s
     * @return
     */
    fun escapeQueryChars(s: String): String {
        if (isEmpty(s)) {
            return s
        }
        val sb = StringBuilder()
        //查询字符串一般不会太长，挨个遍历也花费不了多少时间
        for (element in s) {
            if (element == '\\' || element == '+' || element == '-' || element == '!' || element == '(' || element == ')' || element == ':' || element == '^' || element == '[' || element == ']' || element == '\"' || element == '{' || element == '}' || element == '~' || element == '*' || element == '?' || element == '|' || element == '&' || element == ';' || element == '/' || element == '.' || element == '$' || element == '%' || element == '<' || element == '>' || Character.isWhitespace(
                    element
                )
            ) {
                sb.append('\\')
            }
            sb.append(element)
        }
        return sb.toString()
    }


    /**
     * 获取默认值
     *
     * @param str
     * @param defaultStr
     * @return
     */
    fun getStrWithDefault(str: CharSequence, defaultStr: String?): String? {
        var defaultString = defaultStr
        if (isEmpty(defaultString)) {
            defaultString = ""
        }
        if (isEmpty(str)) {
            return defaultString
        }
        return str.toString()
    }
}