package cn.fw.hestia.rpc.member

import cn.fw.common.util.MobileUtil
import cn.fw.hestia.common.constant.Constant
import cn.fw.hestia.rpc.member.dto.MemberUserDTO
import cn.fw.member.sdk.api.FunctionApi
import cn.fw.member.sdk.api.MemberApi
import kotlinx.coroutines.*
import kotlinx.coroutines.selects.select
import org.slf4j.LoggerFactory
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service

/**
 * 会员服务
 *
 * @author kurisu
 */

@Service
class MemberRpcService
    (
    /**
     * 会员服务
     */
    private val memberApi: MemberApi,
    private val functionApi: FunctionApi
) {
    val logger = LoggerFactory.getLogger(this.javaClass)

    /**
     * 根据会员ID获取会员信息
     *
     * @param userId 会员ID
     * @return 会员信息
     */
    @Cacheable(cacheNames = [Constant.MEMBER_INFO_CACHE], key = "#userId", unless = "#result == null ")
    fun user(userId: Long?): MemberUserDTO? {
        try {
            val msg = memberApi.getMemberById(userId)
            if (!msg.isSuccess) {
                logger.warn("调用Member[根据会员ID[{}]获取会员信息]系统失败, 原因：{}", userId, msg.result)
                return null
            }
            return msg.data?.run {
                MemberUserDTO(
                    id,
                    latestAccountId,
                    realName,
                    credentType,
                    credentNo,
                    birthday = birthDate
                )
            }
        } catch (e: Exception) {
            logger.error("调用Member[根据会员ID[{}]获取会员信息]系统失败", userId, e)
            return null
        }
    }


    /**
     * 查询手机号归属地
     *
     * @param mobile
     * @return
     */
    @Cacheable(cacheNames = ["mobile:attribution"], key = "#mobile", unless = "#result.isEmpty()")
    fun attribution(mobile: String?): String {
        if (mobile.isNullOrEmpty()) {
            return ""
        }
        return runBlocking(Dispatchers.IO) {
            val errHandle = CoroutineExceptionHandler { _, ex ->
                logger.error("查询号码归属地失败:", ex)
            }
            val job = SupervisorJob()
            supervisorScope {
                var att = ""
                launch(job + errHandle) {
                    val j1 = async(job) {
                        val result = MobileUtil.attribution(mobile)
                        result?.run {
                            "$province $city"
                        } ?: ""
                    }
                    val j2 = async(job) {
                        val msg = functionApi.queryMobileLocation(mobile)
                        msg.takeIf { it.isSuccess }?.run { data?.run { "$province $city" } } ?: ""
                    }
                    att = select {
                        j1.onAwait { it }
                        j2.onAwait { it }
                    }
                }.join()
                att
            }
        }
    }
}