package cn.fw.hestia.server.task

import cn.fw.hestia.common.constant.Constant
import cn.fw.hestia.domain.db.MessageHistory
import cn.fw.hestia.domain.enums.MessageStateEnum
import cn.fw.hestia.service.buz.MessageCenterBizService
import cn.fw.hestia.service.data.MessageHistoryService
import com.baomidou.mybatisplus.extension.kotlin.KtQueryWrapper
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component
import org.springframework.util.CollectionUtils
import java.time.LocalDateTime
import java.util.*

/**
 * @author : kurisu
 * @className : FollowNoticeTask
 * @description : 服务号消息定时任务
 * @date: 2020-08-25 16:57
 */
@Component
@ConditionalOnProperty(prefix = "task", name = ["switch"], havingValue = "on")
class SendMessageTask(
    private val messageCenterBizService: MessageCenterBizService,
    private val messageHistoryService: MessageHistoryService,
    private val stringRedisTemplate: StringRedisTemplate,
    @Value("\${spring.cache.custom.global-prefix}:data:clear") private val dataClearKey: String
) {

    /**
     * 发送模板消息
     */
    @Scheduled(initialDelay = 1000 * 10, fixedRate = 1000 * 3)
    fun sendNotice() {
        val list = messageHistoryService.list(
            KtQueryWrapper(MessageHistory::class.java)
                .eq(MessageHistory::state, MessageStateEnum.MADA)
                .eq(MessageHistory::yn, true)
                .lt(MessageHistory::frequency, Constant.MAX_FREQUENCY)
                .lt(MessageHistory::sendTime, Date())
                .last("limit 500")
        )
        if (list.isNullOrEmpty()) {
            return
        }
        list.forEach {
            CoroutineScope(Dispatchers.IO).launch {
                messageCenterBizService.sendMessage(it)
            }
        }
    }

    /**
     * 缓存数据
     */
    @Scheduled(initialDelay = 1000 * 10, fixedRate = 1000 * 10)
    fun cacheOldData() {
        val key: String = dataClearKey
        val setOps = stringRedisTemplate.boundSetOps(key)
        val list = messageHistoryService.list(
            KtQueryWrapper(MessageHistory::class.java)
                .lt(MessageHistory::createTime, LocalDateTime.now().minusMonths(6L))
                .last("limit 1000")
        )
        if (CollectionUtils.isEmpty(list)) {
            return
        }
        list.forEach {
            it.id?.run {
                setOps.add(this.toString())
            }
        }
    }

    /**
     * 清理数据
     */
    @Scheduled(initialDelay = 1000 * 15, fixedRate = 1000 * 15)
    fun clearOldData() {
        val key: String = dataClearKey
        val setOps = stringRedisTemplate.boundSetOps(key)
        var counter = 0
        val removeIds = mutableSetOf<Long>()
        while (counter < 2000) {
            val idStr = setOps.pop() ?: break
            idStr.toLong().run {
                removeIds.add(this)
            }
            counter++
        }
        if (removeIds.isNotEmpty()) {
            messageCenterBizService.clearHistoryBatch(removeIds)
        }
    }
}