package cn.fw.hestia.service.buz

import cn.fw.common.businessvalidator.Validator
import cn.fw.common.data.mybatis.pagination.PageData
import cn.fw.common.page.AppPage
import cn.fw.hestia.common.constant.Constant
import cn.fw.hestia.common.utils.DateUtil
import cn.fw.hestia.common.utils.StringUtils
import cn.fw.hestia.component.SendMsgProducer
import cn.fw.hestia.component.SettingProperty
import cn.fw.hestia.domain.db.MessageHistory
import cn.fw.hestia.domain.db.SendLog
import cn.fw.hestia.domain.enums.MessageStateEnum
import cn.fw.hestia.domain.vo.HistoryQuery
import cn.fw.hestia.domain.vo.MessageHistoryVO
import cn.fw.hestia.rpc.passport.dto.TMParam
import cn.fw.hestia.sdk.params.TemplateMessageParam
import cn.fw.hestia.sdk.result.MessageSendMq
import cn.fw.hestia.sdk.result.SendResult
import cn.fw.hestia.service.data.MessageHistoryService
import cn.fw.hestia.service.data.SendLogService
import cn.fw.passport.sdk.api.param.WxMpTempMessageData
import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.JSONArray
import com.baomidou.mybatisplus.extension.kotlin.KtQueryWrapper
import org.springframework.beans.factory.annotation.Value
import org.springframework.cache.annotation.Cacheable
import org.springframework.data.redis.core.StringRedisTemplate
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.util.CollectionUtils
import java.time.LocalDate
import java.time.LocalDateTime
import java.util.*
import java.util.concurrent.TimeUnit
import java.util.function.Consumer
import java.util.stream.Collectors

/**
 * @author : kurisu
 * @className : MessageCenterBizService
 * @description : 消息中心处理类
 * @date: 2021-09-24 17:10
 */
@Service
class MessageCenterBizService(
    private val messageHistoryService: MessageHistoryService,
    private val sendLogService: SendLogService,
    private val sendMsgProducer: SendMsgProducer,
    private val redisTemplate: StringRedisTemplate,
    private val settingProperty: SettingProperty,
    @Value("\${spring.cache.custom.global-prefix}:template:send:scene:")
    private val prefix: String
) {

    private val sendDataPrefix: String
        get() {
            val today = LocalDate.now().toString()
            return prefix + today
        }

    /**
     * 发送
     *
     * @param templateMessageParam
     * @return token
     */
    @Transactional(rollbackFor = [Exception::class])
    fun saveMessage(templateMessageParam: TemplateMessageParam): SendResult {
        val messageHistory = createBeans(templateMessageParam)
        messageHistoryService.save(messageHistory)
        return SendResult(messageHistory.id, messageHistory.memberId)
    }

    /**
     * 批量发送
     *
     * @param paramList
     * @return token
     */
    @Transactional(rollbackFor = [Exception::class])
    fun saveBatchMessage(paramList: List<TemplateMessageParam>): List<SendResult> {
        val messageHistory = createBeans(paramList)
        messageHistoryService.saveBatch(messageHistory)
        val list: MutableList<SendResult> = ArrayList()
        for ((id, memberId) in messageHistory) {
            list.add(SendResult(id, memberId))
        }
        return list
    }


    /**
     * 发送模板消息
     *
     * @param history
     */
    @Transactional(rollbackFor = [Exception::class])
    fun sendMessage(history: MessageHistory) {
        val key = sendDataPrefix
        val setOps = redisTemplate.boundSetOps(key)
        val seanceToken = history.id.toString()
        if (true == setOps.isMember(seanceToken)) {
            return
        }
        val result = saveLog(createTmParam(history), history.id) { r: SendLog ->
            r.manual = false
            if (true == r.succeed) {
                setOps.add(seanceToken)
            }
        }
        val succeed = Constant.SUCCEED_STR == result
        val frequency = history.frequency!! + 1
        history.frequency = frequency
        if (succeed) {
            history.state = MessageStateEnum.SUMI
            //发送成功发送mq
            sendMsgProducer.send(MessageSendMq(history.id, history.memberId, Date()))
            redisTemplate.expire(key, 24L, TimeUnit.HOURS)
        } else {
            history.sendTime?.let {
                history.sendTime = DateUtil.getExpired(it, 1 shl frequency, Calendar.MINUTE)
            }
        }
        messageHistoryService.updateById(history)
    }


    /**
     * 清理不使用的数据
     *
     * @param id
     */
    @Transactional(rollbackFor = [Exception::class])
    fun clearHistory(id: Long) {
        sendLogService.remove(
            KtQueryWrapper(SendLog::class.java)
                .eq(SendLog::messageId, id)
        )
        messageHistoryService.removeById(id)
    }

    /**
     * 批量清理不使用的数据
     *
     * @param idList
     */
    @Transactional(rollbackFor = [Exception::class])
    fun clearHistoryBatch(idList: Set<Long>) {
        sendLogService.remove(
            KtQueryWrapper(SendLog::class.java)
                .`in`(SendLog::messageId, idList)
        )
        messageHistoryService.removeByIds(idList)
    }

    /**
     * 撤回消息
     *
     * @param sceneToken
     * @return
     */
    @Transactional(rollbackFor = [Exception::class])
    fun revokeMessage(sceneToken: Long?): Boolean {
        val history = messageHistoryService.getById(sceneToken)
        if (Objects.isNull(history)) {
            return true
        }
        history.yn = false
        return messageHistoryService.updateById(history)
    }

    /**
     * 查询未读数
     *
     * @param memberId
     * @return
     */
    @Cacheable(cacheNames = ["UnreadCount"], key = "':unread-count:' + #memberId", condition = "#result != 0")
    fun unreadCount(memberId: Long?): Int {
        if (memberId == null || memberId <= 0) {
            return 0
        }
        return messageHistoryService.count(
            KtQueryWrapper(MessageHistory::class.java)
                .eq(MessageHistory::memberId, memberId)
                .eq(MessageHistory::readz, false)
                .eq(MessageHistory::yn, false)
                .ge(MessageHistory::sendTime, DateUtil.localDateTime2Date(LocalDateTime.now().minusMonths(3L)))
        )
    }

    /**
     * 查询页面所需参数
     *
     * @param sceneToken
     * @return
     */
    @Transactional(rollbackFor = [Exception::class])
    @Cacheable(
        cacheNames = ["ExchangePageParam"],
        key = "':Exchange-Page-Param:' + #sceneToken",
        condition = "#result != null"
    )
    fun queryPageParams(sceneToken: Long?): Map<String, String> {
        val history = messageHistoryService.getById(sceneToken)
        Validator.BV.notNull(history) { "消息不存在或者已经被撤销" }
        readMessage(history)
        val pageParams = history.pageParams
        if (StringUtils.isEmpty(pageParams)) {
            return HashMap(0)
        }
        return JSON.parseObject<HashMap<String, String>>(pageParams, HashMap::class.java)
    }

    /**
     * 查询历史消息记录
     *
     * @param query
     * @return
     */
    fun queryHistory(query: HistoryQuery): AppPage<MessageHistoryVO> {
        if (Objects.isNull(query.memberId) || query.memberId <= 0) {
            return AppPage.empty(query)
        }
        val pageData = messageHistoryService.page<PageData<MessageHistory>>(
            PageData(query), KtQueryWrapper(MessageHistory::class.java)
                .eq(MessageHistory::memberId, query.memberId)
                .eq(MessageHistory::yn, true)
                .ge(
                    MessageHistory::sendTime,
                    DateUtil.localDateTime2Date(LocalDate.now().minusMonths(3L).atStartOfDay())
                )
                .orderByDesc(MessageHistory::createTime)
        )
        val page = AppPage.empty<MessageHistoryVO>(query)
        page.setCurrent(pageData.current.toInt())
        page.setPageSize(pageData.size.toInt())
        page.setTotal(pageData.total)
        page.setData(ArrayList())
        if (!CollectionUtils.isEmpty(pageData.records)) {
            val voList = pageData.records.stream().map { history: MessageHistory? ->
                MessageHistoryVO.with(
                    history
                )
            }.collect(Collectors.toList())
            page.setData(voList)
        }
        return page
    }


    fun readMessage(history: MessageHistory) {
        if (false == history.readz) {
            history.readz = true
            history.state = MessageStateEnum.SUMI
            messageHistoryService.updateById(history)
        }
    }

    fun manualSend(id: Long?): String {
        val messageHistory: MessageHistory? = messageHistoryService.getById(id)
        return messageHistory?.run {
            val result = saveLog(
                createTmParam(messageHistory), messageHistory.id
            ) { r: SendLog -> r.manual = true }
            val succeed = Constant.SUCCEED_STR == result
            if (succeed) {
                messageHistory.state = MessageStateEnum.SUMI
                sendMsgProducer.send(MessageSendMq(messageHistory.id, messageHistory.memberId, Date()))
            }
            messageHistoryService.updateById(messageHistory)
            result
        } ?: "消息不存在"
    }

    val rdsTemplateCode: String
        get() = settingProperty.tempId ?: "无效code"

    private fun saveLog(tmParam: TMParam, messageId: Long?, cons: Consumer<SendLog>): String {
        val result = Constant.SUCCEED_STR
        val sendLog = SendLog(messageId, Date(), succeed = true, manual = false, result)
        sendLogService.save(sendLog)
        return result
    }

    private fun createBeans(paramList: List<TemplateMessageParam>): List<MessageHistory> {
        val list: MutableList<MessageHistory> = ArrayList()
        for (param in paramList) {
            list.add(createBeans(param))
        }
        return list
    }

    private fun createBeans(param: TemplateMessageParam): MessageHistory {
        val memberId = param.memberId
        val content = StringBuilder(param.content)
        if (!CollectionUtils.isEmpty(param.extraMap)) {
            content.append("\n")
            for ((key, value) in param.extraMap) {
                content.append("\n")
                    .append(key)
                    .append("：")
                    .append(value)
            }
        }

        val keywords = listOf(
            WxMpTempMessageData(settingProperty.typeSort1, param.title),
            WxMpTempMessageData(settingProperty.typeSort2, if (StringUtils.isEmpty(param.remark)) "~" else param.remark)
        )

        return MessageHistory(
            id = null,
            memberId = memberId,
            templateCode = settingProperty.tempId,
            title = param.title,
            content = content.toString(),
            keywords = JSONArray.toJSONString(keywords),
            remark = param.remark,
            pagePath = param.path,
            pageParams = if (param.paramMap.isNullOrEmpty()) null else JSON.toJSONString(param.paramMap),
            readz = StringUtils.isEmpty(param.path),
            sendTime = Date(),
            frequency = 0,
            state = MessageStateEnum.MADA,
            yn = true,
            createTime = Date()
        )
    }

    private fun createTmParam(history: MessageHistory): TMParam {
        val (id, memberId, templateCode, title, _, keywords, remark, pagePath) = history
        val tmParam = TMParam(
            memberId,
            title,
            keywords,
            remark,
            path = pagePath,
            sceneToken = id,
            templateCode
        )
        return tmParam
    }
}